@@ -11,14 +11,19 @@ import base64
 from ifconf import configure_module, config_callback
 
 import logging
+logger = logging.getLogger(__name__)
         
 @config_callback
 def config(loader):
     loader.add_attr('hash_algorithm', 'sha1', help='hash algorithm name for session')
     loader.add_attr_list('white_paths', ['/libs/', '/wsd', '/main'], help='paths without authentication')
+    loader.add_attr('stream_key_for_session_opened', 'SESSION/2020', help='stream key for session')
+    loader.add_attr('stream_key_for_session_closed', 'SESSION/CLOSED', help='stream key for session')
+
 
 class Auth(object):
 
+
     def __init__(self, context):
         self.conf = configure_module(config)
         self.context = context
@@ -38,22 +43,30 @@ class Auth(object):
         if not key:
             raise web.HTTPForbidden()
         ssid = self.hash_func(ip.encode()+str(port).encode()+ua.encode()+key.encode()).hexdigest()
-        tsid = await self.context.redis.xadd('SESSION/2020', ip=ip, port=port, ua=ua, ssid=ssid, **query)
+        tsid = await self.context.redis.xadd(self.conf.stream_key_for_session_opened, ip=ip, port=port, ua=ua, ssid=ssid, **query)
         return "{}.{}".format(ssid, base64.b64encode(tsid).decode())
 
     async def check_session(self, request):
-        ssid, tsid = request.match_info['session_id'].split('.')
-        tsid = base64.b64decode(tsid).decode()
+        ssid, tsids = request.match_info['session_id'].split('.')
+        tsid = base64.b64decode(tsids).decode()
         ts = datetime.fromtimestamp(int(tsid.split('-')[0])/1000)
         ip, port = request.transport.get_extra_info('peername')
         ua = request.headers['User-Agent']
-        param = await self.context.redis.xget_str('SESSION/2020', tsid)
+        param = await self.context.redis.xget_str(self.conf.stream_key_for_session_opened, tsid)
         if param['ssid'] != ssid or param['ua'] != ua:
             raise web.HTTPForbidden()
+        return 'SS.'+tsids, param
+    
         #op = int(param['port'])
         #if param['ip'] != ip or port > op+128 or (op > 65535 - 128 and port > 49152 + 128):
         #    raise web.HTTPForbidden()
 
+    async def close_session(self, session_id):
+        tsid = base64.b64decode(session_id[len('SS.'):].encode())
+        param = await self.context.redis.xget_str(self.conf.stream_key_for_session_opened, tsid)
+        if param:
+            await self.context.redis.xadd(self.conf.stream_key_for_session_closed, tsid=tsid, **param)
+
     async def setup(self, app):
         app.middlewares.append(self.middleware)
 
