@@ -1,4 +1,4 @@
-from ducts.event import EventHandler
+from ducts.spi import EventHandler
 
 class Handler(EventHandler):
 
