@@ -1,5 +1,6 @@
 (function( ducts ) {
     ducts.context_url = ducts.context_url || "/ducts";
+    ducts.libs_plugin = ducts.libs_plugin || [];
     ducts.query = '?uuid='+([1e7]+-1e3+-4e3+-8e3+-1e11).replace(/[018]/g, c => (c ^ crypto.getRandomValues(new Uint8Array(1))[0] & 15 >> c / 4).toString(16));
     //ducts.query = '?ts='+Date.now();
     ducts.query += ducts.token ? '&token='+ducts.token : '';
@@ -154,6 +155,8 @@ window.ducts.Duct = class {
 	    () => {return ducts._local.next_rid();};
 	this.open =
 	    () => {return this._open(this);};
+	this.reconnect =
+	    () => {return this._reconnect(this);};
 	this.send = 
 	    (request_id, event_id, data) => {return this._send(this, request_id, event_id, data)};
 	this.close =
@@ -164,12 +167,23 @@ window.ducts.Duct = class {
 	this._event_handler = {};
 	this.setEventHandler = 
 	    (event_id, handler) => {this._event_handler[event_id] = handler;};
-	this.setEventHandler(this.EVENT.ALIVE_MONITORING, (rid, eid, data) => {});
-	
+	this.setEventHandler(this.EVENT.ALIVE_MONITORING, (rid, eid, data) => {
+	    let client_received = new Date().getTime() / 1000;
+	    let server_sent = data[0];
+	    let server_received = data[1];
+	    let client_sent = this._send_timestamp;
+	    //console.log('t0='+client_sent+', t1='+server_received+', t2='+server_sent+',t3='+client_received);
+	    let new_offset = ((server_received - client_sent) - (client_received - server_sent))/2;
+	    let new_latency = ((client_received - client_sent) - (server_sent - server_received))/2;
+	    this.time_offset = (this.time_offset * this._time_count + new_offset) / (this._time_count + 1);
+	    this.time_latency = (this.time_latency * this._time_count + new_latency) / (this._time_count + 1);
+	    this._time_count += 1;
+	    console.log('offset='+this.time_offset+', latency='+this.time_latency);
+	});
 	this.catchall_event_handler = (rid, eid, data) => {};
 	this.uncaught_event_handler = (rid, eid, data) => {};
 	this.event_error_handler = (rid, eid, data, error) => {};
-
+	
     }
 
     get state() {
@@ -180,6 +194,35 @@ window.ducts.Duct = class {
 	}
     }
 
+    _reconnect(self) {
+	return new Promise(function(resolve, reject) {
+	    if (self._ws) {
+		resolve(self);
+		return;
+	    }
+	    let ws = new WebSocket(self.WSD.websocket_url_reconnect);
+	    ws.binaryType = 'arraybuffer';
+	    ws.onopen = 
+		(event) => {
+		    ws.onerror =
+			(event) => {self._connection_listener.onerror(new window.ducts.DuctConnectionEvent('onerror', event));};
+		    ws.onclose =
+			(event) => {self._connection_listener.onclose(new window.ducts.DuctConnectionEvent('onclose', event));};
+		    ws.onmessage = 
+			(event) => {self._onmessage(self, new window.ducts.DuctConnectionEvent('onmessage', event));};
+		    self._ws = ws;
+		    self._onreconnect(self, event);
+		    self._connection_listener.onopen(new window.ducts.DuctConnectionEvent('onopen', event));
+		    resolve(self);
+		};
+	    ws.onerror =
+		(event) => {
+		    self._connection_listener.onerror(new window.ducts.DuctConnectionEvent('onerror', event));
+		    reject(event);
+		};
+	});
+    }
+    
     _open(self) {
 	return new Promise(function(resolve, reject) {
 	    if (self._ws) {
@@ -210,12 +253,20 @@ window.ducts.Duct = class {
     }
     
     _onopen(self, event) {
+	self._send_timestamp = new Date().getTime() / 1000;
+	self.time_offset = 0;
+	self.time_latency = 0;
+	self._time_count = 0;
 	let rid = self.next_rid();
 	let eid = self.EVENT.ALIVE_MONITORING;
-	let value = ''
+	let value = self._send_timestamp;
 	self.send(rid, eid, value);
     }
     
+    _onreconnect(self, event) {
+	console.log('reconnected');
+    }
+    
     _send(self, request_id, event_id, data) {
 	const msgpack = self.encode([request_id, event_id, data])
 	self._ws.send(msgpack)
